@@ -1,6 +1,7 @@
 #include <string.h>
 #include "search.h"
 #include "array.h"
+#include "internal.h"
 
 #define BQ(query) ((BooleanQuery *)(query))
 #define BW(weight) ((BooleanWeight *)(weight))
@@ -110,7 +111,7 @@ static bool dssc_advance_after_current(Scorer *self)
                     break;
                 }
             }
-            top = pq_top(scorer_queue);
+            top = (Scorer *)pq_top(scorer_queue);
             if (top->doc != self->doc) {
                 /* All remaining subscorers are after self->doc */
                 break;
@@ -161,7 +162,7 @@ static bool dssc_skip_to(Scorer *self, int doc_num)
         doc_num = self->doc + 1;
     }
     while (true) { 
-        Scorer *top = pq_top(scorer_queue);
+        Scorer *top = (Scorer *)pq_top(scorer_queue);
         if (top->doc >= doc_num) {
             return dssc_advance_after_current(self);
         }
@@ -280,7 +281,19 @@ typedef struct ConjunctionScorer
 
 static void csc_sort_scorers(ConjunctionScorer *csc)
 {
-    qsort(csc->sub_scorers, csc->ss_cnt, sizeof(Scorer *), &scorer_doc_cmp);
+    int i;
+    Scorer *current = csc->sub_scorers[0], *previous;
+    for (i = 1; i < csc->ss_cnt; i++) {
+        previous = current;
+        current = csc->sub_scorers[i];
+        if (previous->doc > current->doc) {
+            if (!current->skip_to(current, previous->doc)) {
+                csc->more = false;
+                return;
+            }
+        }
+    }
+    /*qsort(csc->sub_scorers, csc->ss_cnt, sizeof(Scorer *), &scorer_doc_cmp);*/
     csc->first_idx = 0;
 }
 
@@ -371,7 +384,7 @@ static bool csc_skip_to(Scorer *self, int doc_num)
     bool more = csc->more;
 
     if (csc->first_time) {
-        csc_init(self, true);
+        csc_init(self, false);
     }
 
     for (i = 0; i < sub_sc_cnt; i++) {
@@ -1255,7 +1268,7 @@ static Weight *bw_new(Query *query, Searcher *searcher)
  *
  ***************************************************************************/
 
-void bc_set_occur(BooleanClause *self, enum BC_TYPE occur)
+void bc_set_occur(BooleanClause *self, BCType occur)
 {
     self->occur = occur;
     switch (occur) {
@@ -1295,7 +1308,7 @@ static int  bc_eq(BooleanClause *self, BooleanClause *o)
     return ((self->occur == o->occur) && q_eq(self->query, o->query)); 
 }
 
-BooleanClause *bc_new(Query *query, enum BC_TYPE occur)
+BooleanClause *bc_new(Query *query, BCType occur)
 {
     BooleanClause *self = ALLOC(BooleanClause);
     self->ref_cnt = 1;
@@ -1402,7 +1415,7 @@ static void bq_extract_terms(Query *self, HashSet *terms)
     }
 }
 
-static char *bq_to_s(Query *self, const char *field)
+static char *bq_to_s(Query *self, Symbol field)
 {
     int i;
     BooleanClause *clause;
@@ -1585,7 +1598,7 @@ BooleanClause *bq_add_clause(Query *self, BooleanClause *bc)
     return bq_add_clause_nr(self, bc);
 }
 
-BooleanClause *bq_add_query_nr(Query *self, Query *sub_query, enum BC_TYPE occur)
+BooleanClause *bq_add_query_nr(Query *self, Query *sub_query, BCType occur)
 {
     BooleanClause *bc;
     if (BQ(self)->clause_cnt >= BQ(self)->max_clause_cnt) {
@@ -1600,7 +1613,7 @@ BooleanClause *bq_add_query_nr(Query *self, Query *sub_query, enum BC_TYPE occur
     return bc;
 }
 
-BooleanClause *bq_add_query(Query *self, Query *sub_query, enum BC_TYPE occur)
+BooleanClause *bq_add_query(Query *self, Query *sub_query, BCType occur)
 {
     REF(sub_query);
     return bq_add_query_nr(self, sub_query, occur);
